﻿using D3StatsClient.Commands;
using D3StatsClient.Model;
using System.Windows;
using System.Windows.Input;
using System.Collections;
using System.Collections.ObjectModel;
using System.Xml.Linq;
using System.Linq;
using System;
using System.Diagnostics;
using System.Timers;
using D3StatsClient.D3MemoryAPI;
using System.Collections.Generic;
using System.Drawing;

namespace D3StatsClient.ViewModel
{

    public class UIItemEntry
    {
        public string Name { get; set; }
        public string Quality { get; set; }
        public string TimeStamp { get; set; }
        public string Level { get; set; }
        public string MagicFind { get; set; }

    }

    public class MainWindowViewModel : ViewModelBase
    {
        public ObservableCollection<UIItemEntry> RecordedItems { get; set; }
        public Stopwatch sw;
        private Timer timer;
        private ItemFinder finder;
        private List<int> _queuedDrops = new List<int>();
        public IStatsProvider provider;

        //Commands
        public ICommand StartRecordingSessionCommand { get; set; }
        public ICommand StopRecordingSessionCommand { get; set; }
        public ICommand OpenSettingsWindowCommand { get; set; }
        public ICommand OpenSessionPickerCommand { get; set; }
        public ICommand ResumeSessionCommand { get; set; }
        public ICommand NewSessionCommand { get; set; }

        public MainWindowViewModel()
        {
            //init commands
            StartRecordingSessionCommand = new StartRecordingSession();
            StopRecordingSessionCommand = new StopRecordingSession();
            OpenSettingsWindowCommand = new OpenSettingsWindow();
            OpenSessionPickerCommand = new OpenSessionPicker();
            ResumeSessionCommand = new ResumeSession();
            NewSessionCommand = new NewSession();

            this.InitViewModel();
            
        }

        public void InitViewModel()
        {
            Sounds.InitSounds();

            Stats = new SessionStats();

            RecordedItems = new ObservableCollection<UIItemEntry>();

            timer = new Timer();
            timer.Enabled = false;
            timer.Interval = 500;
            timer.Elapsed += timer_Elapsed;
        }

        public void EndSession()
        {
            Status = string.Empty;
            SetStatus(false);
            IsRecording = false;
            IsContinuedSession = true;
            IsSessionLoaded = true;
            sw.Stop();
            provider.CloseConnection(sw.Elapsed);
        }

        void timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            try
            {

                string server = D3StatsUtility.GetGameInstanceServerIp();
                int magicFindValue = 0;

                if (finder.Player != null)
                {
                    magicFindValue = (int)Math.Round(finder.Player.GetFloat(Attrib.Magic_Find_Total) * 100);
                }

                if (finder.Follower != null)
                {
                    magicFindValue += (int)Math.Round(finder.Follower.GetFloat(Attrib.Magic_Find_Total) * 100 * 0.20);
                }

                ACDActor acd = finder.FindBestItem(provider);

                if (acd != null)
                {
                    if (acd.ACDType == ACDType.Item)
                    {
                        Item item = new Item(acd);

                        int itemSeed = item.Seed;

                        if (_queuedDrops.Contains(itemSeed))
                        {
                            return;
                        }

                        //We are treating this item, stop processing it.
                        _queuedDrops.Add(itemSeed);


                        //TESTS
                        //for (uint i = 4294963200; i < 4294965000; i++)
                        //{
                        //    //dump all to file
                        //    Logging.Logger.Log(string.Format("{0} -> {1}", i, finder.Player.GetIntInternal(i)));
                        //}

                        ////TESTS
                        //foreach (Attrib attrib in Enum.GetValues(typeof(Attrib)))
                        //{
                        //    //Debug.WriteLine(acd.GetFloat(attrib));
                        //    Debug.WriteLine(string.Format("{0} -> {1}", attrib, finder.Player.GetInt(attrib)));
                        //}

                        ////TESTS
                        //foreach (Attrib attrib in Enum.GetValues(typeof(Attrib)))
                        //{
                        //    //Debug.WriteLine(acd.GetFloat(attrib));
                        //    Debug.WriteLine(string.Format("{0} -> {1}",attrib, acd.GetInt(attrib)));
                        //}

                        //An item has dropped, record it!
                        DropStats stats = new DropStats();
                        stats.Name = item.Name;
                        stats.ItemLevel = item.ItemLevel;  //for some reason ilvl doesnt work on legendaries.
                        stats.Location = finder.LevelArea.ToString();
                        stats.Quality = item.Quality;
                        stats.Seed = itemSeed.ToString();
                        stats.Time = DateTime.Now;
                        stats.MagicFind = magicFindValue;
                        stats.Server = server;

                        stats.Save(provider);

                        Sounds.PlayNotificationSound(stats.Quality, stats.ItemLevel);

                        _queuedDrops.Remove(itemSeed);

                    }
                }

                Dispatcher.BeginInvoke(new Action(() =>
                {
                    this.Server = server;
                }));
                
            }
            catch (Exception ex)
            {
                Logging.Logger.Log(ex.Message);
            }
        }

        public string GetTimeSpanFormattedString(TimeSpan span)
        {
            string formatted = string.Format("{0}{1}{2}{3}",
                  span.Days > 0 ? string.Format("{0:0} days, ", span.Days) : string.Empty,
                  span.Hours > 0 ? string.Format("{0:0} hours, ", span.Hours) : string.Empty,
                  span.Minutes > 0 ? string.Format("{0:0} minutes, ", span.Minutes) : string.Empty,
                  span.Seconds > 0 ? string.Format("{0:0} seconds", span.Seconds) : string.Empty);

            if (formatted.EndsWith(", ")) formatted = formatted.Substring(0, formatted.Length - 2);

            return formatted;
        }

        public bool SetStatus(bool status)
        {
            if (status == true)
            {
                try
                {
                    if (Process == null)
                    {
                        finder = new ItemFinder();
                    }
                    else
                    {
                        finder = new ItemFinder(Process.Id);
                    }
                }
                catch (Exception ex)
                {
                    Logging.Logger.Log(ex.Message);
                    return false;
                }

                timer.Enabled = true;
            }

            if (status == false)
            {
                try
                {
                    finder.Dispose();
                }
                catch (Exception ex)
                {
                    Logging.Logger.Log(ex.Message);
                }

                timer.Enabled = false;
            }

            return true;
        }

        public void ReloadLoadUIData()
        {
            var provider = ((XMLStatsProvider)this.provider).XMLData;

            this.RecordedItems.Clear();

            foreach (XElement e in provider.Descendants("item"))
            {
                AddUIStatEntry(e);
            }

            if (provider.Element("items").Attribute("played") != null)
            {
                this.SessionTime = this.GetTimeSpanFormattedString(TimeSpan.Parse(provider.Element("items").Attribute("played").Value));
            }
            else
            {
                if (provider.Element("items").Attribute("starttime") == null || provider.Element("items").Attribute("endtime") == null)
                {
                    SessionTime = this.GetTimeSpanFormattedString(new TimeSpan(0, 0, 0, 0)); //new TimeSpan().ToString("hh\\:mm\\:ss");
                }
                else
                {
                    var startTime = DateTime.Parse(provider.Element("items").Attribute("starttime").Value);
                    var endTime = DateTime.Parse(provider.Element("items").Attribute("endtime").Value);

                    SessionTime = this.GetTimeSpanFormattedString((endTime - startTime));
                }
            }

            this.XMLData_Changed(null, null);
        }

        private void AddUIStatEntry(XContainer data)
        {
            UIItemEntry entry = new UIItemEntry();
            entry.Level = data.Element("level").Value;
            entry.TimeStamp = DateTime.Parse(data.Element("timestamp").Value).ToLocalTime().ToString();
            entry.Quality = data.Element("quality").Value;
            entry.Name = data.Element("name").Value;
            entry.MagicFind = data.Element("magicfind").Value;

            this.RecordedItems.Add(entry);
        }

        public void XMLData_Changed(object sender, System.Xml.Linq.XObjectChangeEventArgs e)
        {
            Dispatcher.Invoke(new Action(() =>
            {
                XContainer data = sender as XContainer;

                if (data != null)
                {
                    this.AddUIStatEntry(data);
                }

                //Calculate total number of items
                Stats.TotalItems = RecordedItems.Count;
                Stats.ILevel61Number = (from l in RecordedItems where (int.Parse(l.Level) == 61) select l).Count();
                Stats.ILevel62Number = (from l in RecordedItems where (int.Parse(l.Level) == 62) select l).Count();
                Stats.ILevel63Number = (from l in RecordedItems where (int.Parse(l.Level) == 63) select l).Count();
                Stats.Rares = (from l in RecordedItems
                               where
                                   (((ItemQuality)Enum.Parse(typeof(ItemQuality), l.Quality) >= ItemQuality.Rare))
                               select l).Count();

                Stats.Legendaries = (from l in RecordedItems
                                     where
                                         (((ItemQuality)Enum.Parse(typeof(ItemQuality), l.Quality) >= ItemQuality.Legendary))
                                     select l).Count();

                Stats.Magic = (from l in RecordedItems
                               where
                                   (((ItemQuality)Enum.Parse(typeof(ItemQuality), l.Quality) >= ItemQuality.Magic))
                               select l).Count();

                Stats.Set = (from l in RecordedItems
                             where
                                 (((ItemQuality)Enum.Parse(typeof(ItemQuality), l.Quality) >= ItemQuality.Legendary))
                             select l).Count();

                Stats.ILevel63Percentage = ((double)Stats.ILevel63Number / (double)Stats.TotalItems) * 100;

            }));
        }

        #region properties

        public bool DiabloGameIsStarted
        {
            get
            {
                if (Properties.Settings.Default.EnableMultiProcessSupport)
                {
                    return (Process != null);
                }
                else
                {
                    Process[] processes = Process.GetProcessesByName("Diablo III");

                    if (processes != null && processes.Length > 0)
                    {
                        return true;
                    }
                }

                return false;
            }
        }


        public string Server
        {
            get { return (string)GetValue(ServerProperty); }
            set { SetValue(ServerProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Server.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ServerProperty =
            DependencyProperty.Register("Server", typeof(string), typeof(MainWindowViewModel), new PropertyMetadata("N/A"));

        public string SessionName
        {
            get { return (string)GetValue(SessionNameProperty); }
            set { SetValue(SessionNameProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SessionName.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SessionNameProperty =
            DependencyProperty.Register("SessionName", typeof(string), typeof(MainWindowViewModel), new PropertyMetadata(string.Empty));

        

        public Process Process
        {
            get { return (Process)GetValue(ProcessProperty); }
            set { SetValue(ProcessProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Process.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ProcessProperty =
            DependencyProperty.Register("Process", typeof(Process), typeof(MainWindowViewModel), new PropertyMetadata(null));
   

        public int GoldPerHour
        {
            get { return (int)GetValue(GoldPerHourProperty); }
            set { SetValue(GoldPerHourProperty, value); }
        }

        // Using a DependencyProperty as the backing store for GoldPerHour.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty GoldPerHourProperty =
            DependencyProperty.Register("GoldPerHour", typeof(int), typeof(MainWindowViewModel), new PropertyMetadata(0));
       

        public int InitialGoldAmount
        {
            get { return (int)GetValue(InitialGoldAmountProperty); }
            set { SetValue(InitialGoldAmountProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CurrentGoldAmount.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty InitialGoldAmountProperty =
            DependencyProperty.Register("InitialGoldAmount", typeof(int), typeof(MainWindowViewModel), new PropertyMetadata(0));
    

        public bool IsContinuedSession
        {
            get { return (bool)GetValue(IsContinuedSessionProperty); }
            set { SetValue(IsContinuedSessionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsContinuedSession.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsContinuedSessionProperty =
            DependencyProperty.Register("IsContinuedSession", typeof(bool), typeof(MainWindowViewModel), new PropertyMetadata(false));
      
       
        public bool IsSessionLoaded
        {
            get { return (bool)GetValue(IsSessionLoadedProperty); }
            set { SetValue(IsSessionLoadedProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsSessionLoaded.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsSessionLoadedProperty =
            DependencyProperty.Register("IsSessionLoaded", typeof(bool), typeof(MainWindowViewModel), new PropertyMetadata(false));

        public string SessionTime
        {
            get { return (string)GetValue(SessionTimeProperty); }
            set { SetValue(SessionTimeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SessionTime.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SessionTimeProperty =
            DependencyProperty.Register("SessionTime", typeof(string), typeof(MainWindowViewModel), new PropertyMetadata("00:00:00"));


        public SessionStats Stats
        {
            get { return (SessionStats)GetValue(StatsProperty); }
            set { SetValue(StatsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Stats.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StatsProperty =
            DependencyProperty.Register("Stats", typeof(SessionStats), typeof(MainWindowViewModel), new PropertyMetadata(null));
     

        public bool IsRecording
        {
            get { return (bool)GetValue(IsRecordingProperty); }
            set { SetValue(IsRecordingProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsRecording.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsRecordingProperty =
            DependencyProperty.Register("IsRecording", typeof(bool), typeof(MainWindowViewModel), new PropertyMetadata(false));

        public string Status
        {
            get { return (string)GetValue(StatusProperty); }
            set { SetValue(StatusProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Status.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StatusProperty =
            DependencyProperty.Register("Status", typeof(string), typeof(MainWindowViewModel), new PropertyMetadata(string.Empty));

        public string Action
        {
            get { return (string)GetValue(ActionProperty); }
            set { SetValue(ActionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Action.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ActionProperty =
            DependencyProperty.Register("Action", typeof(string), typeof(MainWindowViewModel), new PropertyMetadata(string.Empty));

        #endregion

    }
}
